@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: avrdude.h,v 1.2 2007/01/24 22:43:46 joerg_wunsch Exp $ */
+/* $Id: avrdude.h,v 1.3 2008/11/19 23:32:30 joerg_wunsch Exp $ */
 
 #ifndef avrdude_h
 #define avrdude_h
@@ -46,7 +46,7 @@ extern "C" {
  #define ANTIWARP 3
  #define usleep(us) Sleep(us/1000*ANTIWARP)
 */
-void usleep(unsigned long us);
+int usleep(unsigned int us);
 
 #if !defined(HAVE_GETTIMEOFDAY)
 struct timezone;
