@@ -17,16 +17,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: serbb.h,v 1.1 2005/09/18 20:12:23 joerg_wunsch Exp $ */
+/* $Id: serbb.h,v 1.2 2007/01/24 22:43:46 joerg_wunsch Exp $ */
 
 #ifndef serbb_h
 #define serbb_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 void serbb_initpgm        (PROGRAMMER * pgm);
 
-int serbb_setpin(int fd, int pin, int value);
-int serbb_getpin(int fd, int pin);
-int serbb_highpulsepin(int fd, int pin);
+#ifdef __cplusplus
+}
+#endif
 
 #endif
 
