@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: term.c,v 1.38 2007/01/24 22:43:46 joerg_wunsch Exp $ */
+/* $Id: term.c,v 1.40 2009/02/17 15:31:27 joerg_wunsch Exp $ */
 
 #include "ac_cfg.h"
 
@@ -88,6 +88,11 @@ static int cmd_fosc  (PROGRAMMER * pgm, struct avrpart * p,
 static int cmd_sck   (PROGRAMMER * pgm, struct avrpart * p,
 		      int argc, char *argv[]);
 
+static int cmd_spi   (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
+
+static int cmd_pgm   (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
 struct command cmd[] = {
   { "dump",  cmd_dump,  "dump memory  : %s <memtype> <addr> <N-Bytes>" },
@@ -102,6 +107,8 @@ struct command cmd[] = {
   { "varef", cmd_varef, "set <V[aref]> (STK500 only)" },
   { "fosc",  cmd_fosc,  "set <oscillator frequency> (STK500 only)" },
   { "sck",   cmd_sck,   "set <SCK period> (STK500 only)" },
+  { "spi",   cmd_spi,   "enter direct SPI mode" },
+  { "pgm",   cmd_pgm,   "return to programming mode" },
   { "help",  cmd_help,  "help" },
   { "?",     cmd_help,  "help" },
   { "quit",  cmd_quit,  "quit" }
@@ -111,7 +118,7 @@ struct command cmd[] = {
 
 
 
-
+static int spi_mode = 0;
 
 static int nexttok(char * buf, char ** tok, char ** next)
 {
@@ -445,8 +452,18 @@ static int cmd_send(PROGRAMMER * pgm, struct avrpart * p,
     return -1;
   }
 
-  if (argc != 5) {
-    fprintf(stderr, "Usage: send <byte1> <byte2> <byte3> <byte4>\n");
+  if (spi_mode && (pgm->spi == NULL)) {
+    fprintf(stderr,
+	    "The %s programmer does not support direct SPI transfers.\n",
+	    pgm->type);
+    return -1;
+  }
+
+
+  if ((argc > 5) || ((argc < 5) && (!spi_mode))) {
+    fprintf(stderr, spi_mode?
+      "Usage: send <byte1> [<byte2> [<byte3> [<byte4>]]]\n":
+      "Usage: send <byte1> <byte2> <byte3> <byte4>\n");
     return -1;
   }
 
@@ -465,7 +482,10 @@ static int cmd_send(PROGRAMMER * pgm, struct avrpart * p,
 
   pgm->err_led(pgm, OFF);
 
-  pgm->cmd(pgm, cmd, res);
+  if (spi_mode)
+    pgm->spi(pgm, cmd, res, argc-1);
+  else
+    pgm->cmd(pgm, cmd, res);
 
   /*
    * display results
@@ -662,25 +682,42 @@ static int cmd_varef(PROGRAMMER * pgm, struct avrpart * p,
 		     int argc, char * argv[])
 {
   int rc;
+  unsigned int chan;
   double v;
   char *endp;
 
-  if (argc != 2) {
-    fprintf(stderr, "Usage: varef <value>\n");
+  if (argc != 2 && argc != 3) {
+    fprintf(stderr, "Usage: varef [channel] <value>\n");
     return -1;
   }
-  v = strtod(argv[1], &endp);
-  if (endp == argv[1]) {
-    fprintf(stderr, "%s (varef): can't parse voltage \"%s\"\n",
-            progname, argv[1]);
-    return -1;
+  if (argc == 2) {
+    chan = 0;
+    v = strtod(argv[1], &endp);
+    if (endp == argv[1]) {
+      fprintf(stderr, "%s (varef): can't parse voltage \"%s\"\n",
+              progname, argv[1]);
+      return -1;
+    }
+  } else {
+    chan = strtoul(argv[1], &endp, 10);
+    if (endp == argv[1]) {
+      fprintf(stderr, "%s (varef): can't parse channel \"%s\"\n",
+              progname, argv[1]);
+      return -1;
+    }
+    v = strtod(argv[2], &endp);
+    if (endp == argv[2]) {
+      fprintf(stderr, "%s (varef): can't parse voltage \"%s\"\n",
+              progname, argv[2]);
+      return -1;
+    }
   }
   if (pgm->set_varef == NULL) {
     fprintf(stderr, "%s (varef): the %s programmer cannot set V[aref]\n",
 	    progname, pgm->type);
     return -2;
   }
-  if ((rc = pgm->set_varef(pgm, v)) != 0) {
+  if ((rc = pgm->set_varef(pgm, chan, v)) != 0) {
     fprintf(stderr, "%s (varef): failed to set V[aref] (rc = %d)\n",
 	    progname, rc);
     return -3;
@@ -707,6 +744,22 @@ static int cmd_help(PROGRAMMER * pgm, struct avrpart * p,
   return 0;
 }
 
+static int cmd_spi(PROGRAMMER * pgm, struct avrpart * p,
+        int argc, char * argv[])
+{
+  pgm->setpin(pgm, pgm->pinno[PIN_AVR_RESET], 1);
+  spi_mode = 1;
+  return 0;
+}
+
+static int cmd_pgm(PROGRAMMER * pgm, struct avrpart * p,
+        int argc, char * argv[])
+{
+  pgm->setpin(pgm, pgm->pinno[PIN_AVR_RESET], 0);
+  spi_mode = 0;
+  pgm->initialize(pgm, p);
+  return 0;
+}
 
 static int tokenize(char * s, char *** argv)
 {
